require 'thor'
require 'fileutils'
require 'github_api'

require 'highline/import'
alias :high_ask :ask

CONFIGURATION_FILE = (ENV['HOME'] + '/.octopushrc')

class Octopush < Thor

  desc  "octopush REPOSITORY_NAME",
        "uploads REPOSITORY_NAME to github (creates an empty one if needed)"

  def octopush(name)
    create_repo(name)
    push_to_github
  end

  private

  def create_repo(name)
    initialize_repo(name)
    cd_into_repo(name)
    add_readme
    :done
  end

  def push_to_github
    create_repo_on_github
    add_github_remote
    push_repo

    cyan_arrow
    say "done."

    :done
  end

  def initialize_repo(name = repository_name)
    @repository_name ||= name

    cyan_arrow
    say "Initializing repo in '#{repository_name}'"

    `git init #{repository_name}`
  end


  def cd_into_repo(repository_name)
    Dir.chdir("./#{repository_name}")
    Dir.pwd
  end

  def add_readme
    cyan_arrow
    say "Touching and committing README.md"

    touch_readme
    add_and_commit_readme
    :done
  end

  def touch_readme
    FileUtils.touch('README.md')
  end

  def add_and_commit_readme
    `git add README.md`
    `git commit -m 'add README.md (via octopush)'`
  end

  def create_repo_on_github
    cyan_arrow
    say "Creating repo '#{repository_name}' on GitHub"

    github_user.repos.create(name: repository_name)
  end

  def add_github_remote(remote_name = 'origin')
    cyan_arrow
    say "Adding GitHub as a remote"

    `git remote add #{remote_name} https://github.com/#{username}/#{repository_name}.git`
  end

  def push_repo
    cyan_arrow
    say "Pushing repo to GitHub"

    `git push -u origin master`
  end

  def repository_name
    @repository_name
  end

  def github_user
    @github_user ||= Github.new({
                       login: username,
                       password: password
                     })
  end

  def username
    @username ||= set_username
  end

  def set_username
    if configuration['username'] && configuration['username'] != '<your_username>'
      username = configuration['username']
    else
      username = ask "GitHub Username: "
    end
  end

  def password
    @password ||= set_password
  end

  def set_password
    if configuration['password'] && configuration['password'] != '<your_password>'
      password = configuration['password']
    else
      password = high_ask("GitHub Password: ") { |c| c.echo = '*' }
    end
  end

  def configuration
    begin
      @configuration ||= YAML.load(File.read(CONFIGURATION_FILE))
    rescue
      create_empty_configuration
      return @configuration = {} # an empty configuration..
    end
  end

  def create_empty_configuration
    red_arrow
    say "Configuration not found. Edit your ~/.octopushrc for storing credentials"

    configuration ="username: <your_username>\npassword: <your_password>"
    File.open(CONFIGURATION_FILE, 'wb') { |f| f.write(configuration) }
  end

  def red_arrow
    say '==> ', :red
  end

  def cyan_arrow
    say '==> ', :cyan
  end

end
